"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class DatasetUpdateTypedDict(TypedDict):
    dataset_id: str
    r"""The unique identifier of the dataset being updated"""
    name: NotRequired[str]
    r"""Updated name for the dataset"""
    description: NotRequired[str]
    r"""Updated description for the dataset"""
    datapoints: NotRequired[List[str]]
    r"""Updated list of datapoint ids for the dataset - note the full list is needed"""
    linked_evals: NotRequired[List[str]]
    r"""Updated list of unique evaluation run ids to be associated with this dataset"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Updated metadata to track for the dataset"""


class DatasetUpdate(BaseModel):
    dataset_id: str
    r"""The unique identifier of the dataset being updated"""

    name: Optional[str] = None
    r"""Updated name for the dataset"""

    description: Optional[str] = None
    r"""Updated description for the dataset"""

    datapoints: Optional[List[str]] = None
    r"""Updated list of datapoint ids for the dataset - note the full list is needed"""

    linked_evals: Optional[List[str]] = None
    r"""Updated list of unique evaluation run ids to be associated with this dataset"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Updated metadata to track for the dataset"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class UpdateRunRequestStatus(str, Enum):
    PENDING = "pending"
    COMPLETED = "completed"


class UpdateRunRequestTypedDict(TypedDict):
    event_ids: NotRequired[List[str]]
    r"""Additional sessions/events to associate with this run"""
    dataset_id: NotRequired[str]
    r"""The UUID of the dataset this run is associated with"""
    datapoint_ids: NotRequired[List[str]]
    r"""Additional datapoints to associate with this run"""
    configuration: NotRequired[Dict[str, Any]]
    r"""The configuration being used for this run"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Additional metadata for the run"""
    name: NotRequired[str]
    r"""The name of the run to be displayed"""
    status: NotRequired[UpdateRunRequestStatus]


class UpdateRunRequest(BaseModel):
    event_ids: Optional[List[str]] = None
    r"""Additional sessions/events to associate with this run"""

    dataset_id: Optional[str] = None
    r"""The UUID of the dataset this run is associated with"""

    datapoint_ids: Optional[List[str]] = None
    r"""Additional datapoints to associate with this run"""

    configuration: Optional[Dict[str, Any]] = None
    r"""The configuration being used for this run"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Additional metadata for the run"""

    name: Optional[str] = None
    r"""The name of the run to be displayed"""

    status: Optional[UpdateRunRequestStatus] = None




import os
import sys
from typing import Optional
import yaml
import fnmatch
from .dotdict import dotdict


CONFIG_PATTERN = ["**/honeyhive.yaml"]

def get_assumed_config_path() -> Optional[str]:
    try:
        # Get the directory containing the file being run
        if getattr(sys, 'frozen', False):
            # If the application is run as a bundle, use the sys._MEIPASS
            running_dir = sys._MEIPASS
        else:
            # If it's not bundled, use the directory containing the script
            running_dir = os.path.dirname(os.path.abspath(sys.argv[0]))

        print(running_dir)

        return running_dir

        # Prepend the directory containing the running file to config.path
        config_path = os.path.join(running_dir, USER_CONFIG_PATH)
        if os.path.exists(config_path):
            return config_path
        

    except Exception as e:
        print(f'Error while loading config file: {e}')
        print('Continuing...')
        
    return None

def check_match(path_input, include_patterns, exclude_patterns):
    p = os.path.abspath(path_input)
    if include_patterns:
        include = False
        for pattern in include_patterns:
            if fnmatch.fnmatch(p, pattern):
                include = True
                break
        if not include:
            return False

    if exclude_patterns:
        exclude = False
        for pattern in exclude_patterns:
            if fnmatch.fnmatch(p, pattern):
                exclude = True
                break
        return not exclude

    return True

def collect_files(input_path, include_patterns, exclude_patterns):
    if os.path.isdir(input_path):
        for root, dirs, files in os.walk(input_path):
            for file in files:
                fname = os.path.join(root, file)
                if check_match(fname, include_patterns, exclude_patterns):
                    yield (os.path.dirname(fname), fname)
    else:
        if not check_match(input_path, include_patterns, exclude_patterns):
            print(
                f"Reading {input_path} because it was specified directly. Rename it to *.eval.py "
                + "to include it automatically when you specify a directory."
            )
        yield (os.path.dirname(input_path), input_path)

# TODO: parse evaluators from yaml and add them to the evaluators registry

def load_yaml(_yaml) -> dict:
    if _yaml is None:
        return None
    
    content = None
    try:
        content = yaml.safe_load(_yaml)
    except yaml.YAMLError as e:
        raise ValueError(f"Error parsing YAML: {str(e)}")
    except ValueError as e:
        raise ValueError(f"Validation error: {str(e)}")
        
    return content

def get_yaml_dotdict(path: str | None = None) -> dotdict:
    try:
        if path is None:
            _, path = next(collect_files(os.getcwd(), CONFIG_PATTERN, ["**/site-packages/**"]))
    except StopIteration:
        return dotdict()

    if not path:
        return dotdict()

    with open(path) as f:
        content = load_yaml(f)
    
    content = content or dict()
    
    return dotdict(content)

config = get_yaml_dotdict()



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from honeyhive import utils
from honeyhive._hooks import HookContext
from honeyhive.models import components, errors, operations
from honeyhive.types import BaseModel, OptionalNullable, UNSET
from typing import List, Optional, Union, cast


class Metrics(BaseSDK):
    def get_metrics(
        self,
        *,
        project_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetMetricsResponse:
        r"""Get all metrics

        Retrieve a list of all metrics

        :param project_name: Project name associated with metrics
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetMetricsRequest(
            project_name=project_name,
        )

        req = self.build_request(
            method="GET",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getMetrics",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetMetricsResponse(
                metrics=utils.unmarshal_json(
                    http_res.text, Optional[List[components.Metric]]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_metrics_async(
        self,
        *,
        project_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetMetricsResponse:
        r"""Get all metrics

        Retrieve a list of all metrics

        :param project_name: Project name associated with metrics
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetMetricsRequest(
            project_name=project_name,
        )

        req = self.build_request_async(
            method="GET",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getMetrics",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetMetricsResponse(
                metrics=utils.unmarshal_json(
                    http_res.text, Optional[List[components.Metric]]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def create_metric(
        self,
        *,
        request: Union[components.Metric, components.MetricTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateMetricResponse:
        r"""Create a new metric

        Add a new metric

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.Metric)
        request = cast(components.Metric, request)

        req = self.build_request(
            method="POST",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.Metric
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.CreateMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_metric_async(
        self,
        *,
        request: Union[components.Metric, components.MetricTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateMetricResponse:
        r"""Create a new metric

        Add a new metric

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.Metric)
        request = cast(components.Metric, request)

        req = self.build_request_async(
            method="POST",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.Metric
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.CreateMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def update_metric(
        self,
        *,
        request: Union[components.MetricEdit, components.MetricEditTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateMetricResponse:
        r"""Update an existing metric

        Edit a metric

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.MetricEdit)
        request = cast(components.MetricEdit, request)

        req = self.build_request(
            method="PUT",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.MetricEdit
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="updateMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def update_metric_async(
        self,
        *,
        request: Union[components.MetricEdit, components.MetricEditTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateMetricResponse:
        r"""Update an existing metric

        Edit a metric

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.MetricEdit)
        request = cast(components.MetricEdit, request)

        req = self.build_request_async(
            method="PUT",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.MetricEdit
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="updateMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.UpdateMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def delete_metric(
        self,
        *,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteMetricResponse:
        r"""Delete a metric

        Remove a metric

        :param metric_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteMetricRequest(
            metric_id=metric_id,
        )

        req = self.build_request(
            method="DELETE",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="deleteMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.DeleteMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def delete_metric_async(
        self,
        *,
        metric_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteMetricResponse:
        r"""Delete a metric

        Remove a metric

        :param metric_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteMetricRequest(
            metric_id=metric_id,
        )

        req = self.build_request_async(
            method="DELETE",
            path="/metrics",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="deleteMetric",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return operations.DeleteMetricResponse(
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import (
    sessionstartrequest as components_sessionstartrequest,
)
from honeyhive.types import BaseModel
import httpx
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class StartSessionRequestBodyTypedDict(TypedDict):
    session: NotRequired[components_sessionstartrequest.SessionStartRequestTypedDict]


class StartSessionRequestBody(BaseModel):
    session: Optional[components_sessionstartrequest.SessionStartRequest] = None


class StartSessionResponseBodyTypedDict(TypedDict):
    r"""Session successfully started"""

    session_id: NotRequired[str]


class StartSessionResponseBody(BaseModel):
    r"""Session successfully started"""

    session_id: Optional[str] = None


class StartSessionResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[StartSessionResponseBodyTypedDict]
    r"""Session successfully started"""


class StartSessionResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[StartSessionResponseBody] = None
    r"""Session successfully started"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.models.components import dataset as components_dataset
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, QueryParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class Type(str, Enum):
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """

    EVALUATION = "evaluation"
    FINE_TUNING = "fine-tuning"


class GetDatasetsRequestTypedDict(TypedDict):
    project: str
    r"""Project Name associated with the datasets like `New Project`"""
    type: NotRequired[Type]
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """
    dataset_id: NotRequired[str]
    r"""Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`"""


class GetDatasetsRequest(BaseModel):
    project: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Project Name associated with the datasets like `New Project`"""

    type: Annotated[
        Optional[Type],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Type of the dataset - \"evaluation\" or \"fine-tuning\" """

    dataset_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Unique dataset ID for filtering specific dataset like `663876ec4611c47f4970f0c3`"""


class GetDatasetsResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    testcases: NotRequired[List[components_dataset.DatasetTypedDict]]


class GetDatasetsResponseBody(BaseModel):
    r"""Successful response"""

    testcases: Optional[List[components_dataset.Dataset]] = None


class GetDatasetsResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetDatasetsResponseBodyTypedDict]
    r"""Successful response"""


class GetDatasetsResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetDatasetsResponseBody] = None
    r"""Successful response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from enum import Enum
from honeyhive.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class ResultsTypedDict(TypedDict):
    r"""The results of the evaluation (including pass/fails and metric aggregations)"""


class Results(BaseModel):
    r"""The results of the evaluation (including pass/fails and metric aggregations)"""


class EvaluationRunStatus(str, Enum):
    PENDING = "pending"
    COMPLETED = "completed"


class EvaluationRunTypedDict(TypedDict):
    run_id: NotRequired[str]
    project: NotRequired[str]
    r"""The UUID of the project this run is associated with"""
    created_at: NotRequired[datetime]
    r"""The date and time the run was created"""
    event_ids: NotRequired[List[str]]
    r"""The UUIDs of the sessions/events this run is associated with"""
    dataset_id: NotRequired[Nullable[str]]
    r"""The UUID of the dataset this run is associated with"""
    datapoint_ids: NotRequired[List[str]]
    r"""The UUIDs of the datapoints from the original dataset this run is associated with"""
    results: NotRequired[ResultsTypedDict]
    r"""The results of the evaluation (including pass/fails and metric aggregations)"""
    configuration: NotRequired[Dict[str, Any]]
    r"""The configuration being used for this run"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Additional metadata for the run"""
    status: NotRequired[EvaluationRunStatus]
    name: NotRequired[str]
    r"""The name of the run to be displayed"""


class EvaluationRun(BaseModel):
    run_id: Optional[str] = None

    project: Optional[str] = None
    r"""The UUID of the project this run is associated with"""

    created_at: Optional[datetime] = None
    r"""The date and time the run was created"""

    event_ids: Optional[List[str]] = None
    r"""The UUIDs of the sessions/events this run is associated with"""

    dataset_id: OptionalNullable[str] = UNSET
    r"""The UUID of the dataset this run is associated with"""

    datapoint_ids: Optional[List[str]] = None
    r"""The UUIDs of the datapoints from the original dataset this run is associated with"""

    results: Optional[Results] = None
    r"""The results of the evaluation (including pass/fails and metric aggregations)"""

    configuration: Optional[Dict[str, Any]] = None
    r"""The configuration being used for this run"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Additional metadata for the run"""

    status: Optional[EvaluationRunStatus] = None

    name: Optional[str] = None
    r"""The name of the run to be displayed"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "run_id",
            "project",
            "created_at",
            "event_ids",
            "dataset_id",
            "datapoint_ids",
            "results",
            "configuration",
            "metadata",
            "status",
            "name",
        ]
        nullable_fields = ["dataset_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



import asyncio
from asyncio import futures
from timeit import default_timer
from typing import Collection

from wrapt import wrap_function_wrapper as _wrap

from opentelemetry.instrumentation.instrumentor import BaseInstrumentor
from opentelemetry.instrumentation.utils import unwrap
from opentelemetry.metrics import get_meter
from opentelemetry.trace import get_tracer
from opentelemetry.trace.status import Status, StatusCode


ASYNCIO_PREFIX = "asyncio"
VERSION = "1.0.0"

# TODO: this instrumentor does not work as expected

class AsyncioInstrumentor(BaseInstrumentor):
    """
    A simplified instrumentor for asyncio that wraps and traces all main asyncio methods.
    """

    # List of asyncio methods to instrument (excluding 'gather')
    methods_to_instrument = [
        "create_task",
        "ensure_future",
        "wait",
        "wait_for",
        "as_completed",
        "to_thread",
        "run_coroutine_threadsafe",
    ]

    # List of specific coroutines to instrument
    coroutines_to_instrument = [
        # "sleep",
    ]

    def instrumentation_dependencies(self) -> Collection[str]:
        return []

    def _instrument(self, **kwargs):
        # Initialize tracer and meter
        self._tracer = get_tracer(
            __name__, VERSION, tracer_provider=kwargs.get("tracer_provider")
        )
        self._meter = get_meter(
            __name__, VERSION, meter_provider=kwargs.get("meter_provider")
        )

        # Create metrics
        self.process_duration_histogram = self._meter.create_histogram(
            name="asyncio.process.duration",
            description="Duration of asyncio process",
            unit="ms",
        )
        self.process_created_counter = self._meter.create_counter(
            name="asyncio.process.created",
            description="Number of asyncio processes",
            unit="1",
        )

        # Instrument each specified asyncio method (excluding 'gather')
        for method in self.methods_to_instrument:
            self._instrument_method(method)

        # Instrument 'gather' with a separate wrapper
        self._instrument_gather()

        # Instrument specific coroutines
        for coro in self.coroutines_to_instrument:
            self._instrument_coroutine(coro)

    def _uninstrument(self, **kwargs):
        # Uninstrument each specified asyncio method
        for method in self.methods_to_instrument:
            unwrap(asyncio, method)

        # Uninstrument 'gather'
        unwrap(asyncio, "gather")

        # Uninstrument specific coroutines
        for coro in self.coroutines_to_instrument:
            self._uninstrument_coroutine(coro)

    def _instrument_method(self, method_name: str):
        """
        Wrap and trace the specified asyncio method.
        """

        def wrapper(wrapped, instance, args, kwargs):
            start_time = default_timer()
            span = self._tracer.start_span(f"{ASYNCIO_PREFIX}.{method_name}")

            try:
                result = wrapped(*args, **kwargs)
                # If the result is a coroutine or future, attach a callback to trace its completion
                if asyncio.iscoroutine(result) or futures.isfuture(result):
                    self._attach_callback(result, span, start_time)
                return result
            except Exception as exc:
                span.set_status(Status(StatusCode.ERROR, str(exc)))
                raise
            finally:
                span.end()

        _wrap(asyncio, method_name, wrapper)

    def _instrument_gather(self):
        """
        Wrap and trace asyncio.gather with a separate span that acts as the parent for all gathered coroutines.
        """

        original_gather = asyncio.gather

        async def gather_wrapper(*args, **kwargs):
            with self._tracer.start_as_current_span(
                f"{ASYNCIO_PREFIX}.gather"
            ) as span:
                start_time = default_timer()
                try:
                    result = await original_gather(*args, **kwargs)
                    span.set_status(Status(StatusCode.OK))
                    return result
                except Exception as exc:
                    span.set_status(Status(StatusCode.ERROR, str(exc)))
                    raise
                finally:
                    duration = max(default_timer() - start_time, 0)
                    self.process_duration_histogram.record(
                        duration, {"operation": ASYNCIO_PREFIX}
                    )
                    self.process_created_counter.add(
                        1, {"operation": ASYNCIO_PREFIX}
                    )

        # Replace asyncio.gather with the gather_wrapper
        _wrap(
            asyncio, 
            "gather", 
            lambda wrapped, instance, args, kwargs: \
                gather_wrapper(*args, **kwargs)
        )

    def _attach_callback(self, obj, span, start_time):
        """
        Attach a callback to the coroutine or future to record metrics upon completion.
        """

        def callback(fut):
            duration = max(default_timer() - start_time, 0)
            self.process_duration_histogram.record(duration, {"operation": ASYNCIO_PREFIX})
            self.process_created_counter.add(1, {"operation": ASYNCIO_PREFIX})

            if fut.cancelled():
                span.set_status(Status(StatusCode.ERROR, "Cancelled"))
            elif fut.exception():
                span.set_status(Status(StatusCode.ERROR, str(fut.exception())))
            else:
                span.set_status(Status(StatusCode.OK))
            span.end()

        if asyncio.iscoroutine(obj):
            obj = asyncio.ensure_future(obj)
        obj.add_done_callback(callback)

    def _instrument_coroutine(self, coro_name: str):
        """
        Wrap and trace the specified asyncio coroutine.
        """

        original_coro = getattr(asyncio, coro_name, None)
        if original_coro is None or not asyncio.iscoroutinefunction(original_coro):
            # The specified coroutine does not exist or is not a coroutine function
            return

        def coro_wrapper(wrapped, instance, args, kwargs):
            async def traced_coroutine(*args, **kwargs):
                span = self._tracer.start_span(f"{ASYNCIO_PREFIX}.{coro_name}")
                start_time = default_timer()
                try:
                    result = await wrapped(*args, **kwargs)
                    span.set_status(Status(StatusCode.OK))
                    return result
                except Exception as exc:
                    span.set_status(Status(StatusCode.ERROR, str(exc)))
                    raise
                finally:
                    duration = max(default_timer() - start_time, 0)
                    self.process_duration_histogram.record(
                        duration, {"operation": ASYNCIO_PREFIX, "coroutine": coro_name}
                    )
                    self.process_created_counter.add(
                        1, {"operation": ASYNCIO_PREFIX, "coroutine": coro_name}
                    )
                    span.end()

            return traced_coroutine(*args, **kwargs)

        _wrap(asyncio, coro_name, coro_wrapper)

    def _uninstrument_coroutine(self, coro_name: str):
        """
        Unwrap the specified asyncio coroutine.
        """
        original_coro = getattr(asyncio, coro_name, None)
        if original_coro is None or not asyncio.iscoroutinefunction(original_coro):
            return
        unwrap(asyncio, coro_name)


instrumentor = AsyncioInstrumentor()
instrumentor.instrument()



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import NotRequired


class CreateDatapointRequestTypedDict(TypedDict):
    project: str
    r"""Name for the project to which the datapoint belongs"""
    inputs: Dict[str, Any]
    r"""Arbitrary JSON object containing the inputs for the datapoint"""
    history: NotRequired[List[Dict[str, Any]]]
    r"""Conversation history associated with the datapoint"""
    ground_truth: NotRequired[Dict[str, Any]]
    r"""Expected output JSON object for the datapoint"""
    linked_event: NotRequired[str]
    r"""Event id for the event from which the datapoint was created"""
    linked_datasets: NotRequired[List[str]]
    r"""Ids of all datasets that include the datapoint"""
    metadata: NotRequired[Dict[str, Any]]
    r"""Any additional metadata for the datapoint"""


class CreateDatapointRequest(BaseModel):
    project: str
    r"""Name for the project to which the datapoint belongs"""

    inputs: Dict[str, Any]
    r"""Arbitrary JSON object containing the inputs for the datapoint"""

    history: Optional[List[Dict[str, Any]]] = None
    r"""Conversation history associated with the datapoint"""

    ground_truth: Optional[Dict[str, Any]] = None
    r"""Expected output JSON object for the datapoint"""

    linked_event: Optional[str] = None
    r"""Event id for the event from which the datapoint was created"""

    linked_datasets: Optional[List[str]] = None
    r"""Ids of all datasets that include the datapoint"""

    metadata: Optional[Dict[str, Any]] = None
    r"""Any additional metadata for the datapoint"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .createeventbatch import (
    CreateEventBatchResponseBody,
    CreateEventBatchResponseBodyData,
)
from .createmodeleventbatch import (
    CreateModelEventBatchResponseBody,
    CreateModelEventBatchResponseBodyData,
)
from .sdkerror import SDKError

__all__ = [
    "CreateEventBatchResponseBody",
    "CreateEventBatchResponseBodyData",
    "CreateModelEventBatchResponseBody",
    "CreateModelEventBatchResponseBodyData",
    "SDKError",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class Operator(str, Enum):
    r"""The type of filter you are performing - \"is\", \"is not\", \"contains\", \"not contains\", \"greater than\" """

    IS = "is"
    IS_NOT = "is not"
    CONTAINS = "contains"
    NOT_CONTAINS = "not contains"
    GREATER_THAN = "greater than"


class Type(str, Enum):
    r"""The data type you are using - \"string\", \"number\", \"boolean\", \"id\" (for object ids)"""

    STRING = "string"
    NUMBER = "number"
    BOOLEAN = "boolean"
    ID = "id"


class EventFilterTypedDict(TypedDict):
    field: NotRequired[str]
    r"""The field name that you are filtering by like `metadata.cost`, `inputs.chat_history.0.content`"""
    value: NotRequired[str]
    r"""The value that you are filtering the field for"""
    operator: NotRequired[Operator]
    r"""The type of filter you are performing - \"is\", \"is not\", \"contains\", \"not contains\", \"greater than\" """
    type: NotRequired[Type]
    r"""The data type you are using - \"string\", \"number\", \"boolean\", \"id\" (for object ids)"""


class EventFilter(BaseModel):
    field: Optional[str] = None
    r"""The field name that you are filtering by like `metadata.cost`, `inputs.chat_history.0.content`"""

    value: Optional[str] = None
    r"""The value that you are filtering the field for"""

    operator: Optional[Operator] = None
    r"""The type of filter you are performing - \"is\", \"is not\", \"contains\", \"not contains\", \"greater than\" """

    type: Optional[Type] = None
    r"""The data type you are using - \"string\", \"number\", \"boolean\", \"id\" (for object ids)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from honeyhive.types import BaseModel
import pydantic
from pydantic import ConfigDict
from typing import Any, Dict, List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class PutConfigurationRequestCallType(str, Enum):
    r"""Type of API calling - \"chat\" or \"completion\" """

    CHAT = "chat"
    COMPLETION = "completion"


class PutConfigurationRequestResponseFormatTypedDict(TypedDict):
    r"""Response format for the model with the key \"type\" and value \"text\" or \"json_object\" """


class PutConfigurationRequestResponseFormat(BaseModel):
    r"""Response format for the model with the key \"type\" and value \"text\" or \"json_object\" """


class PutConfigurationRequestSelectedFunctionsTypedDict(TypedDict):
    id: NotRequired[str]
    r"""UUID of the function"""
    name: NotRequired[str]
    r"""Name of the function"""
    description: NotRequired[str]
    r"""Description of the function"""
    parameters: NotRequired[Dict[str, Any]]
    r"""Parameters for the function"""


class PutConfigurationRequestSelectedFunctions(BaseModel):
    id: Optional[str] = None
    r"""UUID of the function"""

    name: Optional[str] = None
    r"""Name of the function"""

    description: Optional[str] = None
    r"""Description of the function"""

    parameters: Optional[Dict[str, Any]] = None
    r"""Parameters for the function"""


class PutConfigurationRequestFunctionCallParams(str, Enum):
    r"""Function calling mode - \"none\", \"auto\" or \"force\" """

    NONE = "none"
    AUTO = "auto"
    FORCE = "force"


class PutConfigurationRequestParametersTypedDict(TypedDict):
    call_type: PutConfigurationRequestCallType
    r"""Type of API calling - \"chat\" or \"completion\" """
    model: str
    r"""Model unique name"""
    hyperparameters: NotRequired[Dict[str, Any]]
    r"""Model-specific hyperparameters"""
    response_format: NotRequired[PutConfigurationRequestResponseFormatTypedDict]
    r"""Response format for the model with the key \"type\" and value \"text\" or \"json_object\" """
    selected_functions: NotRequired[
        List[PutConfigurationRequestSelectedFunctionsTypedDict]
    ]
    r"""List of functions to be called by the model, refer to OpenAI schema for more details"""
    function_call_params: NotRequired[PutConfigurationRequestFunctionCallParams]
    r"""Function calling mode - \"none\", \"auto\" or \"force\" """
    force_function: NotRequired[Dict[str, Any]]
    r"""Force function-specific parameters"""


class PutConfigurationRequestParameters(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    call_type: PutConfigurationRequestCallType
    r"""Type of API calling - \"chat\" or \"completion\" """

    model: str
    r"""Model unique name"""

    hyperparameters: Optional[Dict[str, Any]] = None
    r"""Model-specific hyperparameters"""

    response_format: Annotated[
        Optional[PutConfigurationRequestResponseFormat],
        pydantic.Field(alias="responseFormat"),
    ] = None
    r"""Response format for the model with the key \"type\" and value \"text\" or \"json_object\" """

    selected_functions: Annotated[
        Optional[List[PutConfigurationRequestSelectedFunctions]],
        pydantic.Field(alias="selectedFunctions"),
    ] = None
    r"""List of functions to be called by the model, refer to OpenAI schema for more details"""

    function_call_params: Annotated[
        Optional[PutConfigurationRequestFunctionCallParams],
        pydantic.Field(alias="functionCallParams"),
    ] = None
    r"""Function calling mode - \"none\", \"auto\" or \"force\" """

    force_function: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="forceFunction")
    ] = None
    r"""Force function-specific parameters"""

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]


class PutConfigurationRequestEnv(str, Enum):
    DEV = "dev"
    STAGING = "staging"
    PROD = "prod"


class PutConfigurationRequestType(str, Enum):
    r"""Type of the configuration - \"LLM\" or \"pipeline\" - \"LLM\" by default"""

    LLM = "LLM"
    PIPELINE = "pipeline"


class PutConfigurationRequestTypedDict(TypedDict):
    project: str
    r"""Name of the project to which this configuration belongs"""
    name: str
    r"""Name of the configuration"""
    provider: str
    r"""Name of the provider - \"openai\", \"anthropic\", etc."""
    parameters: PutConfigurationRequestParametersTypedDict
    env: NotRequired[List[PutConfigurationRequestEnv]]
    r"""List of environments where the configuration is active"""
    type: NotRequired[PutConfigurationRequestType]
    r"""Type of the configuration - \"LLM\" or \"pipeline\" - \"LLM\" by default"""
    user_properties: NotRequired[Dict[str, Any]]
    r"""Details of user who created the configuration"""


class PutConfigurationRequest(BaseModel):
    project: str
    r"""Name of the project to which this configuration belongs"""

    name: str
    r"""Name of the configuration"""

    provider: str
    r"""Name of the provider - \"openai\", \"anthropic\", etc."""

    parameters: PutConfigurationRequestParameters

    env: Optional[List[PutConfigurationRequestEnv]] = None
    r"""List of environments where the configuration is active"""

    type: Optional[PutConfigurationRequestType] = None
    r"""Type of the configuration - \"LLM\" or \"pipeline\" - \"LLM\" by default"""

    user_properties: Optional[Dict[str, Any]] = None
    r"""Details of user who created the configuration"""



import copy
import jinja2

from typing import Any

class TemplateString(str):
    def render(self, **kwargs):
        return jinja2.Template(self).render(**kwargs)

class dotdict(dict):
    def __init__(self, d=None):
        # TODO: handle YAMLs that are not dictionaries
        try:
            super().__init__({} if d is None else d)
        except Exception as e:
            raise ValueError(f"Error initializing dotdict. Ensure that the YAML is a valid dictionary.")

    @staticmethod
    def _is_dict(value):
        return isinstance(value, dict) and not isinstance(value, dotdict)
    
    def __getattr__(self, key) -> jinja2.Template | dict | list | Any:
        if key.startswith('__') and key.endswith('__'):
            return super().__getattr__(key)
        try:
            if key in self:
                value = self[key]
                if dotdict._is_dict(value):
                    value = dotdict(value)
                    self[key] = value
                elif isinstance(value, list):
                    value = [
                        dotdict(item) if dotdict._is_dict(item) else item
                        for item in value
                    ]
                    self[key] = value
                
                # make string renderable
                if isinstance(value, str):
                    try:
                        value = TemplateString(value)
                    except Exception as e:
                        pass

                return value
            return super().__getattr__(key)
        except KeyError:
            raise AttributeError(f"'{type(self).__name__}' object has no attribute or key '{key}'")

    def __setattr__(self, key, value):
        if key.startswith('__') and key.endswith('__'):
            super().__setattr__(key, value)
        else:
            self[key] = value

    def __getitem__(self, key):
        value = super().__getitem__(key)
        if dotdict._is_dict(value):
            value = dotdict(value)
            self[key] = value
        return value

    def __setitem__(self, key, value):
        # Convert nested dictionaries in lists
        if isinstance(value, list):
            value = [
                dotdict(item) if isinstance(item, dict) and not isinstance(item, dotdict)
                else item
                for item in value
            ]
        # Convert direct dictionary values
        elif isinstance(value, dict) and not isinstance(value, dotdict):
            value = dotdict(value)
        super().__setitem__(key, value)

    def __delattr__(self, key):
        if key.startswith('__') and key.endswith('__'):
            super().__delattr__(key)
        else:
            del self[key]

    def __eq__(self, other):
        if isinstance(other, (dict, dotdict)):
            return dict(self) == dict(other)
        elif isinstance(other, list):
            # If we're comparing with a list, get the value from self
            # and compare it with the list
            for key, value in self.items():
                if value == other:
                    return True
            return False
        return other == self

    def __deepcopy__(self, memo):
        return dotdict(copy.deepcopy(dict(self), memo))



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from honeyhive import utils
from honeyhive._hooks import HookContext
from honeyhive.models import components, errors, operations
from honeyhive.types import BaseModel, OptionalNullable, UNSET
from typing import Optional, Union, cast


class Experiments(BaseSDK):
    def create_run(
        self,
        *,
        request: Union[
            components.CreateRunRequest, components.CreateRunRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateRunResponse:
        r"""Create a new evaluation run

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.CreateRunRequest)
        request = cast(components.CreateRunRequest, request)

        req = self.build_request(
            method="POST",
            path="/runs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CreateRunRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="createRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateRunResponse(
                create_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.CreateRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_run_async(
        self,
        *,
        request: Union[
            components.CreateRunRequest, components.CreateRunRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.CreateRunResponse:
        r"""Create a new evaluation run

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, components.CreateRunRequest)
        request = cast(components.CreateRunRequest, request)

        req = self.build_request_async(
            method="POST",
            path="/runs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CreateRunRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="createRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateRunResponse(
                create_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.CreateRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_runs(
        self,
        *,
        project: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetRunsResponse:
        r"""Get a list of evaluation runs

        :param project:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetRunsRequest(
            project=project,
        )

        req = self.build_request(
            method="GET",
            path="/runs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getRuns",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetRunsResponse(
                get_runs_response=utils.unmarshal_json(
                    http_res.text, Optional[components.GetRunsResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_runs_async(
        self,
        *,
        project: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetRunsResponse:
        r"""Get a list of evaluation runs

        :param project:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetRunsRequest(
            project=project,
        )

        req = self.build_request_async(
            method="GET",
            path="/runs",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getRuns",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetRunsResponse(
                get_runs_response=utils.unmarshal_json(
                    http_res.text, Optional[components.GetRunsResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_run(
        self,
        *,
        run_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetRunResponse:
        r"""Get details of an evaluation run

        :param run_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetRunRequest(
            run_id=run_id,
        )

        req = self.build_request(
            method="GET",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetRunResponse(
                get_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.GetRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_run_async(
        self,
        *,
        run_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetRunResponse:
        r"""Get details of an evaluation run

        :param run_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetRunRequest(
            run_id=run_id,
        )

        req = self.build_request_async(
            method="GET",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetRunResponse(
                get_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.GetRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def update_run(
        self,
        *,
        run_id: str,
        update_run_request: Union[
            components.UpdateRunRequest, components.UpdateRunRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateRunResponse:
        r"""Update an evaluation run

        :param run_id:
        :param update_run_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.UpdateRunRequest(
            run_id=run_id,
            update_run_request=utils.get_pydantic_model(
                update_run_request, components.UpdateRunRequest
            ),
        )

        req = self.build_request(
            method="PUT",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_run_request,
                False,
                False,
                "json",
                components.UpdateRunRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="updateRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateRunResponse(
                update_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.UpdateRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def update_run_async(
        self,
        *,
        run_id: str,
        update_run_request: Union[
            components.UpdateRunRequest, components.UpdateRunRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.UpdateRunResponse:
        r"""Update an evaluation run

        :param run_id:
        :param update_run_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.UpdateRunRequest(
            run_id=run_id,
            update_run_request=utils.get_pydantic_model(
                update_run_request, components.UpdateRunRequest
            ),
        )

        req = self.build_request_async(
            method="PUT",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_run_request,
                False,
                False,
                "json",
                components.UpdateRunRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="updateRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdateRunResponse(
                update_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.UpdateRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def delete_run(
        self,
        *,
        run_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteRunResponse:
        r"""Delete an evaluation run

        :param run_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteRunRequest(
            run_id=run_id,
        )

        req = self.build_request(
            method="DELETE",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="deleteRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.DeleteRunResponse(
                delete_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.DeleteRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def delete_run_async(
        self,
        *,
        run_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.DeleteRunResponse:
        r"""Delete an evaluation run

        :param run_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.DeleteRunRequest(
            run_id=run_id,
        )

        req = self.build_request_async(
            method="DELETE",
            path="/runs/{run_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="deleteRun",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.DeleteRunResponse(
                delete_run_response=utils.unmarshal_json(
                    http_res.text, Optional[components.DeleteRunResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_experiment_result(
        self,
        *,
        run_id: str,
        project_id: str,
        aggregate_function: Optional[operations.AggregateFunction] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetExperimentResultResponse:
        r"""Retrieve experiment result

        :param run_id:
        :param project_id:
        :param aggregate_function:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetExperimentResultRequest(
            run_id=run_id,
            project_id=project_id,
            aggregate_function=aggregate_function,
        )

        req = self.build_request(
            method="GET",
            path="/runs/{run_id}/result",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getExperimentResult",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetExperimentResultResponse(
                experiment_result_response=utils.unmarshal_json(
                    http_res.text, Optional[components.ExperimentResultResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_experiment_result_async(
        self,
        *,
        run_id: str,
        project_id: str,
        aggregate_function: Optional[operations.AggregateFunction] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetExperimentResultResponse:
        r"""Retrieve experiment result

        :param run_id:
        :param project_id:
        :param aggregate_function:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetExperimentResultRequest(
            run_id=run_id,
            project_id=project_id,
            aggregate_function=aggregate_function,
        )

        req = self.build_request_async(
            method="GET",
            path="/runs/{run_id}/result",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getExperimentResult",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetExperimentResultResponse(
                experiment_result_response=utils.unmarshal_json(
                    http_res.text, Optional[components.ExperimentResultResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_experiment_comparison(
        self,
        *,
        run_id_1: str,
        run_id_2: str,
        project_id: str,
        aggregate_function: Optional[operations.QueryParamAggregateFunction] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetExperimentComparisonResponse:
        r"""Retrieve experiment comparison

        :param run_id_1:
        :param run_id_2:
        :param project_id:
        :param aggregate_function:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetExperimentComparisonRequest(
            run_id_1=run_id_1,
            run_id_2=run_id_2,
            project_id=project_id,
            aggregate_function=aggregate_function,
        )

        req = self.build_request(
            method="GET",
            path="/runs/{run_id_1}/compare-with/{run_id_2}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getExperimentComparison",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetExperimentComparisonResponse(
                experiment_comparison_response=utils.unmarshal_json(
                    http_res.text, Optional[components.ExperimentComparisonResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_experiment_comparison_async(
        self,
        *,
        run_id_1: str,
        run_id_2: str,
        project_id: str,
        aggregate_function: Optional[operations.QueryParamAggregateFunction] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.GetExperimentComparisonResponse:
        r"""Retrieve experiment comparison

        :param run_id_1:
        :param run_id_2:
        :param project_id:
        :param aggregate_function:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.GetExperimentComparisonRequest(
            run_id_1=run_id_1,
            run_id_2=run_id_2,
            project_id=project_id,
            aggregate_function=aggregate_function,
        )

        req = self.build_request_async(
            method="GET",
            path="/runs/{run_id_1}/compare-with/{run_id_2}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getExperimentComparison",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetExperimentComparisonResponse(
                experiment_comparison_response=utils.unmarshal_json(
                    http_res.text, Optional[components.ExperimentComparisonResponse]
                ),
                status_code=http_res.status_code,
                content_type=http_res.headers.get("Content-Type") or "",
                raw_response=http_res,
            )
        if utils.match_response(http_res, ["400", "4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
import httpx
from typing import Any, Dict, Optional, TypedDict
from typing_extensions import NotRequired


class UpdateEventRequestBodyTypedDict(TypedDict):
    event_id: str
    metadata: NotRequired[Dict[str, Any]]
    feedback: NotRequired[Dict[str, Any]]
    metrics: NotRequired[Dict[str, Any]]
    outputs: NotRequired[Dict[str, Any]]
    config: NotRequired[Dict[str, Any]]
    user_properties: NotRequired[Dict[str, Any]]
    duration: NotRequired[float]


class UpdateEventRequestBody(BaseModel):
    event_id: str

    metadata: Optional[Dict[str, Any]] = None

    feedback: Optional[Dict[str, Any]] = None

    metrics: Optional[Dict[str, Any]] = None

    outputs: Optional[Dict[str, Any]] = None

    config: Optional[Dict[str, Any]] = None

    user_properties: Optional[Dict[str, Any]] = None

    duration: Optional[float] = None


class UpdateEventResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""


class UpdateEventResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.models.components import datapoint as components_datapoint
from honeyhive.types import BaseModel
from honeyhive.utils import FieldMetadata, PathParamMetadata
import httpx
from typing import List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class GetDatapointRequestTypedDict(TypedDict):
    id: str
    r"""Datapoint ID like `65c13dbbd65fb876b7886cdb`"""


class GetDatapointRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Datapoint ID like `65c13dbbd65fb876b7886cdb`"""


class GetDatapointResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    datapoint: NotRequired[List[components_datapoint.DatapointTypedDict]]


class GetDatapointResponseBody(BaseModel):
    r"""Successful response"""

    datapoint: Optional[List[components_datapoint.Datapoint]] = None


class GetDatapointResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetDatapointResponseBodyTypedDict]
    r"""Successful response"""


class GetDatapointResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetDatapointResponseBody] = None
    r"""Successful response"""



import uuid
from traceback import print_exc
import os
import sys
import threading
import io
from contextlib import redirect_stdout

from honeyhive.utils.telemetry import Telemetry
from honeyhive.utils.baggage_dict import BaggageDict
from honeyhive.models import operations, components, errors
from honeyhive.sdk import HoneyHive

from traceloop.sdk import Traceloop
from traceloop.sdk.tracing.tracing import TracerWrapper

from opentelemetry import context, baggage
from opentelemetry.sdk.metrics.export import ConsoleMetricExporter
from opentelemetry.propagators.composite import CompositePropagator
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator
from opentelemetry.baggage.propagation import W3CBaggagePropagator

DEFAULT_SERVER_URL = "https://api.honeyhive.ai"

class HoneyHiveTracer:
    
    # static variables
    verbose = False
    _is_traceloop_initialized = False
    api_key = None
    is_evaluation = False
    instrumentation_id = None
    instance = None
    server_url = None

    def __init__(
        self,
        api_key=None,
        project=None,
        session_name=None,
        source=None,
        server_url=None,
        session_id=None,
        disable_http_tracing=False,
        disable_batch=False,
        verbose=False,
        inputs=None,
        is_evaluation=False,
        run_id=None,
        dataset_id=None,
        datapoint_id=None,
        link_carrier=None
    ):
        try:
            # is_evaluation
            if HoneyHiveTracer.is_evaluation:
                # If we're in an evaluation, only new evaluate sessions are allowed
                if not is_evaluation:
                    raise Exception("Evaluation sessions can only be started by the evaluate function.")
            
            # api_key
            if HoneyHiveTracer.api_key is None:
                if api_key is None:
                    # get and validate api key from os env
                    env_api_key = os.getenv("HH_API_KEY")
                    if not HoneyHiveTracer._validate_api_key(env_api_key):
                        raise Exception("api_key must be specified or set in environment variable HH_API_KEY.")
                    api_key = env_api_key
                else:
                    # validate user-provided api key
                    if not HoneyHiveTracer._validate_api_key(api_key):
                        raise Exception("api_key must be a string.")
                    
                # set api key
                HoneyHiveTracer.api_key = api_key
            
            # server_url
            if HoneyHiveTracer.server_url is None:
                if server_url is None:
                    # get server url from os env with default
                    env_server_url = os.getenv("HH_API_URL", DEFAULT_SERVER_URL)
                    if not HoneyHiveTracer._validate_server_url(env_server_url):
                        raise Exception("Invalid server URL in environment variable HH_API_URL.")
                    server_url = env_server_url
                else:
                    # validate user-provided server url
                    if not HoneyHiveTracer._validate_server_url(server_url):
                        raise Exception("server_url must be a valid URL string.")
                # set server url
                HoneyHiveTracer.server_url = server_url
            
            # project
            if project is None:
                project = os.getenv("HH_PROJECT")
                if project is None:
                    raise Exception("project must be specified or set in environment variable HH_PROJECT.")
            
            # session_name
            if session_name is None:
                try:
                    session_name = os.path.basename(sys.argv[0])
                except Exception as e:
                    if HoneyHiveTracer.verbose:
                        print(f"Error setting session_name: {e}")
                    session_name = "unknown"

            # source
            if source is None:
                source = os.getenv("HH_SOURCE", "dev")
            
            # verbose
            HoneyHiveTracer.verbose = verbose
            
            # TODO: migrate to log-based session initialization
            # self.session_id = str(uuid.uuid4()).upper()
            if session_id is None:
                self.session_id = HoneyHiveTracer.__start_session(
                    api_key, project, session_name, source, server_url, inputs
                )
            else:
                # Validate that session_id is a valid UUID
                try:
                    uuid.UUID(session_id)
                    self.session_id = session_id.lower()
                except (ValueError, AttributeError, TypeError):
                    raise errors.SDKError("session_id must be a valid UUID string.")

            # baggage
            self.baggage = BaggageDict().update({
                "session_id": self.session_id,
                "project": project,
                "source": source,
                "disable_http_tracing": str(disable_http_tracing).lower(),
            })

            # evaluation
            if is_evaluation:
                self.baggage.update({
                    "run_id": run_id,
                    "dataset_id": dataset_id,
                    "datapoint_id": datapoint_id,
                })

            # Initialize the Composite Propagator
            HoneyHiveTracer.propagator = CompositePropagator(
                propagators=[
                    TraceContextTextMapPropagator(),
                    W3CBaggagePropagator()
                ]
            )

            # instrument tracer with lock
            with threading.Lock():
                # Initialize Traceloop with CompositePropagator
                if not HoneyHiveTracer._is_traceloop_initialized:
                    # Temporarily redirect stdout to suppress Traceloop init messages
                    with redirect_stdout(io.StringIO()):
                        Traceloop.init(
                            api_endpoint=f"{HoneyHiveTracer.server_url}/opentelemetry",
                            api_key=HoneyHiveTracer.api_key,
                            metrics_exporter=ConsoleMetricExporter(out=open(os.devnull, "w")),
                            disable_batch=disable_batch,
                            propagator=HoneyHiveTracer.propagator
                        )
                    # Print initialization message in orange color (works in both bash and Windows)
                    print("\033[38;5;208mHoneyHive is initialized\033[0m")
                    HoneyHiveTracer._is_traceloop_initialized = True
                    HoneyHiveTracer.instrumentation_id = str(uuid.uuid4()).upper()
                    HoneyHiveTracer.is_evaluation = is_evaluation
                Telemetry().capture("tracer_init", {"hhai_session_id": self.session_id, "hhai_project": project})
                    
            # link_carrier
            if link_carrier is not None:
                self.link(link_carrier)
            else:
                # attach baggage to the current context
                ctx = context.get_current() # deep copy of the current context
                ctx = self.baggage.set_all_baggage(ctx)
                context.attach(ctx)
            
            # traceloop sets "association_properties" in the context
            # however it is not propagated since it doesn't follow the W3C spec for Baggage
            # since traceloop stamps "association_properties" from the context into every span when it starts, 
            # we must attach the baggage in traceloop format as well
            # Traceloop.set_baggage_properties(self.baggage)
            Traceloop.set_association_properties(self.baggage)

            # save the instance
            HoneyHiveTracer.instance = self
            
            # ------------------------------------------------------------
            # TODO: log-based session initialization
            # ------------------------------------------------------------
            # save the init metadata
            # self._init_metadata = {
            #     "project": project,
            #     "session_name": session_name,
            #     "source": source,
            #     "server_url": server_url,
            #     "verbose": verbose,
            #     "disable_batch": disable_batch,
            #     "link_carrier_provided": link_carrier is not None,
            #     "instrumentation_id": HoneyHiveTracer.instrumentation_id,
            # }
            
            # # log the session initialization
            # @trace
            # def __session_init():
            #     enrich_span(metadata={
            #         '_init_metadata': self._init_metadata
            #     })
            # __session_init()
            # ------------------------------------------------------------
        except errors.SDKError as e:
            # Raise SDK exceptions as-is but without traceback
            print(f"\033[91mHoneyHive SDK Error: {str(e)}\033[0m")
        except:
            # Log other exceptions if verbose is enabled
            if HoneyHiveTracer.verbose:
                print_exc()
            else:
                pass


    # TODO: remove this, legacy DX
    @staticmethod
    def init(*args, **kwargs):
        HoneyHiveTracer.instance = HoneyHiveTracer(*args, **kwargs)
        return HoneyHiveTracer.instance
    
    @staticmethod
    def _validate_api_key(api_key):
        return api_key and type(api_key) == str
    
    @staticmethod
    def _validate_server_url(server_url):
        return server_url and type(server_url) == str
    
    @staticmethod
    def __start_session(api_key, project, session_name, source, server_url, inputs=None):
        sdk = HoneyHive(bearer_auth=api_key, server_url=server_url)
        res = sdk.session.start_session(
            request=operations.StartSessionRequestBody(
                session=components.SessionStartRequest(
                    project=project,
                    session_name=session_name,
                    source=source,
                    inputs=inputs or {},
                )
            )
        )
        assert res.status_code == 200, f"Failed to start session: {res.raw_response.text}"
        assert res.object.session_id is not None, "Failure initializing session"
        return res.object.session_id
    
    def _sanitize_carrier(carrier, getter):
        # check for baggage in the headers, potentially re-cased
        _propagation_carrier = {}
        for key in ['baggage', 'traceparent']:
            carrier_value = \
                getter.get(carrier, key.lower()) or \
                getter.get(carrier, key.capitalize()) or \
                getter.get(carrier, key.upper())
            if carrier_value is not None:
                _propagation_carrier[key] = [carrier_value]
        return _propagation_carrier
    
    def link(self, carrier={}, getter=BaggageDict.DefaultGetter):
        ctx = context.get_current() # deep copy of the current context
        
        # extract baggage from the carrier
        carrier = HoneyHiveTracer._sanitize_carrier(carrier, getter)
        ctx = HoneyHiveTracer.propagator.extract(carrier, ctx, getter=getter)
        
        # attach the baggage to the current context
        token = context.attach(ctx)
        
        # current context should now have baggage and span context from the carrier
        # it has been fully linked to the parent context
        
        # update the Traceloop baggage in the current context
        # this will be stamped on every span in this context
        bags = self.baggage.get_all_baggage()
        Traceloop.set_association_properties(bags)
        
        return token
    
    def unlink(self, token):
        # included for completeness, but not necessary
        context.detach(token)
        bags = self.baggage.get_all_baggage()
        Traceloop.set_association_properties(bags)
    
    def inject(self, carrier={}, setter=BaggageDict.DefaultSetter):
        # inject current trace and baggage context into the carrier
        HoneyHiveTracer.propagator.inject(carrier, None, setter)
        return carrier

    @staticmethod
    def flush():
        if HoneyHiveTracer._is_traceloop_initialized:
            TracerWrapper().flush()
        else:
            print("\033[91mCould not flush: HoneyHiveTracer not initialized successfully\033[0m")

    def enrich_session(
        self,
        session_id=None,
        metadata=None, 
        feedback=None, 
        metrics=None, 
        config=None, 
        inputs=None, 
        outputs=None, 
        user_properties=None
    ):
        # TODO: migrate to log-based session enrichments
        # @trace
        # def __enrich_session():
        #     _enrichments = {}
        #     if metadata is not None:
        #         _enrichments["metadata"] = metadata
        #     if feedback is not None:
        #         _enrichments["feedback"] = feedback
        #     if metrics is not None:
        #         _enrichments["metrics"] = metrics
        #     if config is not None:
        #         _enrichments["config"] = config
        #     if inputs is not None:
        #         _enrichments["inputs"] = inputs
        #     if outputs is not None:
        #         _enrichments["outputs"] = outputs
        #     if user_properties is not None:
        #         _enrichments["user_properties"] = user_properties
        #     enrich_span(metadata={
        #         '_enrichments': _enrichments,
        #         '_init_metadata': self._init_metadata
        #     })
        # __enrich_session()
        if not HoneyHiveTracer._is_traceloop_initialized:
            print("\033[91mCould not enrich session: HoneyHiveTracer not initialized successfully\033[0m")
            return
        
        session_id = session_id or self.session_id
        try:
            sdk = HoneyHive(bearer_auth=HoneyHiveTracer.api_key, server_url=HoneyHiveTracer.server_url)
            update_request = operations.UpdateEventRequestBody(event_id=session_id)
            if feedback is not None:
                update_request.feedback = feedback
            if metrics is not None:
                update_request.metrics = metrics
            if metadata is not None:
                update_request.metadata = metadata
            if config is not None:
                update_request.config = config
            if inputs is not None:
                print('inputs are not supported in enrich_session') # TODO: add support for inputs (type change)
            if outputs is not None:
                update_request.outputs = outputs
            if user_properties is not None:
                update_request.user_properties = user_properties
            response: operations.UpdateEventResponse = sdk.events.update_event(request=update_request)
            if response.status_code != 200:
                raise Exception(f"Failed to enrich session: {response.raw_response.text}")
        except:
            if HoneyHiveTracer.verbose:
                print_exc()
            else:
                pass


def enrich_session(
    session_id=None,
    metadata=None,
    feedback=None,
    metrics=None,
    config=None,
    inputs=None,
    outputs=None,
    user_properties=None
):
    if not HoneyHiveTracer._is_traceloop_initialized:
        print("\033[91mCould not enrich session: HoneyHiveTracer not initialized successfully\033[0m")
        return
    try:
        sdk = HoneyHive(bearer_auth=HoneyHiveTracer.api_key, server_url=HoneyHiveTracer.instance.server_url)
        if session_id is None and HoneyHiveTracer.instance is None:
            raise Exception("Please initialize HoneyHiveTracer before calling enrich_session")
        session_id = session_id or HoneyHiveTracer.instance.session_id
        update_request = operations.UpdateEventRequestBody(event_id=session_id)
        if feedback is not None:
            update_request.feedback = feedback
        if metrics is not None:
            update_request.metrics = metrics
        if metadata is not None:
            update_request.metadata = metadata
        if config is not None:
            update_request.config = config
        if inputs is not None:
            print('inputs are not supported in enrich_session') # TODO: add support for inputs (type change)
        if outputs is not None:
            update_request.outputs = outputs
        if user_properties is not None:
            update_request.user_properties = user_properties
        response: operations.UpdateEventResponse = sdk.events.update_event(request=update_request)
        if response.status_code != 200:
            raise Exception(f"Failed to enrich session: {response.raw_response.text}")
    except:
        if HoneyHiveTracer.verbose:
            print_exc()
        else:
            pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from honeyhive.types import BaseModel
import httpx
import pydantic
from typing import Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class CreateDatapointResultTypedDict(TypedDict):
    inserted_id: NotRequired[str]


class CreateDatapointResult(BaseModel):
    inserted_id: Annotated[Optional[str], pydantic.Field(alias="insertedId")] = None


class CreateDatapointResponseBodyTypedDict(TypedDict):
    r"""Datapoint successfully created"""

    result: NotRequired[CreateDatapointResultTypedDict]


class CreateDatapointResponseBody(BaseModel):
    r"""Datapoint successfully created"""

    result: Optional[CreateDatapointResult] = None


class CreateDatapointResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[CreateDatapointResponseBodyTypedDict]
    r"""Datapoint successfully created"""


class CreateDatapointResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[CreateDatapointResponseBody] = None
    r"""Datapoint successfully created"""
